@@ -11,7 +11,7 @@ use strict;
 use vars qw($VERSION);
 use Carp ();
 
-$VERSION = '1.23';
+$VERSION = '1.26';
 
 sub PV  { 0 }
 sub IV  { 1 }
@@ -26,6 +26,7 @@ my $ERRORS = {
         1000 => "INI - constructor failed",
         1001 => "sep_char is equal to quote_char or escape_char",
         1002 => "INI - allow_whitespace with escape_char or quote_char SP or TAB",
+        1003 => "INI - \r or \n in main attr not allowed",
 
         2010 => "ECR - QUO char inside quotes followed by CR not part of EOL",
         2011 => "ECR - Characters after end of quoted field",
@@ -84,6 +85,8 @@ my %def_attr = (
     blank_is_undef      => 0,
     empty_is_undef      => 0,
     auto_diag           => 0,
+    quote_space         => 1,
+    quote_null          => 1,
 
     _EOF                => 0,
     _STATUS             => undef,
@@ -137,6 +140,28 @@ sub version {
 ################################################################################
 # new
 ################################################################################
+
+sub _check_sanity {
+    my ( $self ) = @_;
+
+    for ( qw( sep_char quote_char escape_char ) ) {
+        ( exists $self->{$_} && defined $self->{$_} && $self->{$_} =~ m/[\r\n]/ ) and return 1003;
+    }
+
+    if ( $self->{allow_whitespace} and
+           ( defined $self->{quote_char}  && $self->{quote_char}  =~ m/^[ \t]$/ ) 
+           ||
+           ( defined $self->{escape_char} && $self->{escape_char} =~ m/^[ \t]$/ )
+    ) {
+       #$last_new_error = "INI - allow_whitespace with escape_char or quote_char SP or TAB";
+       #$last_new_err_num = 1002;
+       return 1002;
+    }
+
+    return 0;
+}
+
+
 sub new {
     my $proto = shift;
     my $attr  = @_ > 0 ? shift : {};
@@ -152,19 +177,19 @@ sub new {
     for my $prop (keys %$attr) { # if invalid attr, return undef
         unless ($prop =~ /^[a-z]/ && exists $def_attr{$prop}) {
             $last_new_error = "INI - Unknown attribute '$prop'";
+            error_diag() if $attr->{ auto_diag };
             return;
         }
         $self->{$prop} = $attr->{$prop};
     }
 
-    if ( $self->{allow_whitespace} and
-           ( defined $self->{quote_char}  && $self->{quote_char}  =~ m/^[ \t]$/ ) 
-           ||
-           ( defined $self->{escape_char} && $self->{escape_char} =~ m/^[ \t]$/ )
-    ) {
-       $last_new_error = "INI - allow_whitespace with escape_char or quote_char SP or TAB";
-       $last_new_err_num = 1002;
-       return;
+    my $ec = _check_sanity( $self );
+
+    if ( $ec ) {
+        $last_new_error   = $ERRORS->{ $ec };
+        $last_new_err_num = $ec;
+        return;
+        #$class->SetDiag ($ec);
     }
 
     $last_new_error = '';
@@ -211,7 +236,8 @@ sub error_diag {
     unless (defined $context) { # Void context
         if ( $diag[0] ) {
             my $msg = "# CSV_PP ERROR: " . $diag[0] . " - $diag[1]\n";
-            $self->{auto_diag} > 1 ? die $msg : warn $msg;
+            ref $self ? ( $self->{auto_diag} > 1 ? die $msg : warn $msg )
+                      : warn $msg;
         }
         return;
     }
@@ -247,15 +273,15 @@ sub _combine {
     $self->{_STRING}      = '';
     $self->{_STATUS}      = 0;
 
-    my ($always_quote, $binary, $quot, $sep, $esc, $empty_is_undef)
-            = @{$self}{qw/always_quote binary quote_char sep_char escape_char empty_is_undef/};
+    my ($always_quote, $binary, $quot, $sep, $esc, $empty_is_undef, $quote_space, $quote_null)
+            = @{$self}{qw/always_quote binary quote_char sep_char escape_char empty_is_undef quote_space quote_null/};
 
     if(!defined $quot){ $quot = ''; }
 
     return $self->_set_error_diag(1001) if ($sep eq $esc or $sep eq $quot);
 
     my $re_esc = $self->{_re_comb_escape}->{$quot}->{$esc} ||= qr/(\Q$quot\E|\Q$esc\E)/;
-    my $re_sp  = $self->{_re_comb_sp}->{$sep}              ||= qr/[\s\Q$sep\E]/;
+    my $re_sp  = $self->{_re_comb_sp}->{$sep}->{$quote_space} ||= ( $quote_space ? qr/[\s\Q$sep\E]/ : qr/[\Q$sep\E]/ );
 
     my $must_be_quoted;
     for my $column (@part) {
@@ -284,14 +310,11 @@ sub _combine {
             $must_be_quoted++;
         }
 
-        if($binary){
+        if( $binary and $quote_null ){
             use bytes;
             $must_be_quoted++ if ( $column =~ s/\0/${esc}0/g || $column =~ /[\x00-\x1f\x7f-\xa0]/ );
         }
 
-        if ( $empty_is_undef and defined $column and not length $column ) {
-        }
-        #elsif($always_quote or $must_be_quoted){
         if($always_quote or $must_be_quoted){
             $column = $quot . $column . $quot;
         }
@@ -322,9 +345,11 @@ sub _parse {
             qw/binary quote_char sep_char escape_char types keep_meta_info allow_whitespace eol blank_is_undef empty_is_undef/
            };
 
-    $sep  = "\0" unless (defined $sep);
+    $sep  = ',' unless (defined $sep);
     $esc  = "\0" unless (defined $esc);
-    $quot = ''   unless (defined $quot);
+    $quot = "\0" unless (defined $quot);
+
+    my $quot_is_null = $quot eq "\0"; # in this case, any fields are not interpreted as quoted data.
 
     return $self->_set_error_diag(1001) if (($sep eq $esc or $sep eq $quot) and $sep ne "\0");
 
@@ -379,7 +404,7 @@ sub _parse {
         $pos += length $col;
 
         if ( ( !$binary and !$utf8 ) and $col =~ /[^\x09\x20-\x7E]/) { # Binary character, binary off
-            if ( $col =~ $re_quoted ) {
+            if ( not $quot_is_null and $col =~ $re_quoted ) {
                 $self->_set_error_diag(
                       $col =~ /\n([^\n]*)/ ? (2021, $pos - 1 - length $1)
                     : $col =~ /\r([^\r]*)/ ? (2022, $pos - 1 - length $1)
@@ -405,16 +430,17 @@ sub _parse {
             last;
         }
 
-        if ($col =~ $re_quoted) {
+        if ( not $quot_is_null and $col =~ $re_quoted ) {
             $flag |= IS_QUOTED if ($keep_meta_info);
             $col = $1;
 
-            my $flga_in_quot_esp;
+            my $flag_in_quot_esp;
             while ( $col =~ /$re_in_quot_esp1/g ) {
                 my $str = $1;
-                $flga_in_quot_esp = 1;
+                $flag_in_quot_esp = 1;
 
                 if ($str !~ $re_in_quot_esp2) {
+
                     unless ($self->{allow_loose_escapes}) {
                         $self->_set_error_diag( 2025, $pos - 2 ); # Needless ESC in quoted field
                         $palatable = 0;
@@ -429,7 +455,7 @@ sub _parse {
 
             last unless ( $palatable );
 
-            unless ( $flga_in_quot_esp ) {
+            unless ( $flag_in_quot_esp ) {
                 if ($col =~ /(?<!\Q$esc\E)\Q$esc\E/) {
                     $self->_set_error_diag( 4002, $pos - 1 ); # No escaped ESC in quoted field
                     $palatable = 0;
@@ -451,7 +477,7 @@ sub _parse {
 
         # quoted but invalid
 
-        elsif ($col =~ $re_invalid_quot) {
+        elsif ( not $quot_is_null and $col =~ $re_invalid_quot ) {
 
             unless ($self->{allow_loose_quotes} and $col =~ /$re_quot_char/) {
                 $self->_set_error_diag(
@@ -594,7 +620,7 @@ sub getline {
 
     my $quot = $self->{quote_char};
     my $sep  = $self->{sep_char};
-    my $re   = qr/(?:\Q$quot\E)/;
+    my $re   = defined $quot ? qr/(?:\Q$quot\E)/ : '';
 
     local $/ = "\r" if $self->{_AUTO_DETECT_CR};
 
@@ -807,8 +833,8 @@ sub _set_error_diag {
 ################################################################################
 
 BEGIN {
-    for my $method ( qw/sep_char always_quote binary keep_meta_info allow_loose_quotes allow_loose_escapes
-                            verbatim blank_is_undef empty_is_undef auto_diag/ ) {
+    for my $method ( qw/always_quote binary keep_meta_info allow_loose_quotes allow_loose_escapes
+                            verbatim blank_is_undef empty_is_undef auto_diag quote_space quote_null/ ) {
         eval qq|
             sub $method {
                 \$_[0]->{$method} = defined \$_[1] ? \$_[1] : 0 if (\@_ > 1);
@@ -819,12 +845,24 @@ BEGIN {
 }
 
 
+
+sub sep_char {
+    my $self = shift;
+    if ( @_ ) {
+        $self->{sep_char} = $_[0];
+        my $ec = _check_sanity( $self );
+        $ec and Carp::croak( $self->SetDiag( $ec ) );
+    }
+    $self->{sep_char};
+}
+
+
 sub quote_char {
     my $self = shift;
     if ( @_ ) {
-        my $qc = shift;
-        defined $qc && $qc =~ m/^[ \t]$/ && $self->{allow_whitespace} and Carp::croak( $self->SetDiag(1002) );
-        $self->{quote_char} = $qc;
+        $self->{quote_char} = $_[0];
+        my $ec = _check_sanity( $self );
+        $ec and Carp::croak( $self->SetDiag( $ec ) );
     }
     $self->{quote_char};
 }
@@ -833,9 +871,9 @@ sub quote_char {
 sub escape_char {
     my $self = shift;
     if ( @_ ) {
-        my $es = shift;
-        defined $es && $es =~ m/^[ \t]$/ && $self->{allow_whitespace} and Carp::croak( $self->SetDiag(1002) );
-        $self->{escape_char} = $es;
+        $self->{escape_char} = $_[0];
+        my $ec = _check_sanity( $self );
+        $ec and Carp::croak( $self->SetDiag( $ec ) );
     }
     $self->{escape_char};
 }
@@ -943,7 +981,7 @@ is a XS module and Text::CSV_PP is a Puer Perl one.
 
 =head1 VERSION
 
-    1.22
+    1.24
 
 This module is compatible with Text::CSV_XS B<0.68> and later.
 
@@ -1135,7 +1173,7 @@ be escaped as C<"0>.) By default this feature is off.
 If a string is marked UTF8, binary will be turned on automatically when
 binary characters other than CR or NL are encountered. Note that a simple
 string like C<"\x{00a0}"> might still be binary, but not marked UTF8, so
-setting C<{ binary => 1 }> is still a wise option.
+setting C<{ binary =E<gt> 1 }> is still a wise option.
 
 =item types
 
@@ -1151,6 +1189,20 @@ example, if they contain the separator. If you set this attribute to
 a TRUE value, then all fields will be quoted. This is typically easier
 to handle in external applications.
 
+=item quote_space
+
+By default, a space in a field would trigger quotation. As no rule
+exists this to be forced in CSV, nor any for the opposite, the default
+is true for safety. You can exclude the space from this trigger by
+setting this option to 0.
+
+=item quote_null
+
+By default, a NULL byte in a field would be escaped. This attribute
+enables you to treat the NULL byte as a simple binary character in
+binary mode (the C<{ binary =E<gt> 1 }> is set). The default is true.
+You can prevent NULL escapes by setting this attribute to 0.
+
 =item keep_meta_info
 
 By default, the parsing of input lines is as simple and fast as
@@ -1218,6 +1270,8 @@ is equivalent to
      sep_char            => ',',
      eol                 => $\,
      always_quote        => 0,
+     quote_space         => 1,
+     quote_null          => 1,
      binary              => 0,
      keep_meta_info      => 0,
      allow_loose_quotes  => 0,
@@ -1551,6 +1605,11 @@ or the escape character, as that will invalidate all parsing rules.
 Using C<allow_whitespace> when either C<escape_char> or C<quote_char> is
 equal to SPACE or TAB is too ambiguous to allow.
 
+=item 1003 "INI - \r or \n in main attr not allowed"
+
+Using default C<eol> characters in either C<sep_char>, C<quote_char>, or
+C<escape_char> is not allowed.
+
 =item 2010 "ECR - QUO char inside quotes followed by CR not part of EOL"
 
 =item 2011 "ECR - Characters after end of quoted field"
@@ -1613,7 +1672,7 @@ Text::CSV was written by E<lt>alan[at]mfgrtl.comE<gt>.
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2005-2009 by Makamaka Hannyaharamitu, E<lt>makamaka[at]cpan.orgE<gt>
+Copyright 2005-2010 by Makamaka Hannyaharamitu, E<lt>makamaka[at]cpan.orgE<gt>
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself. 
